from apm import *
import numpy as np
import random

def sim_init(s,a):
   apm(s,a,'clear all')
   apm_load(s,a,'process.apm')
   csv_load(s,a,'process.csv')
   apm_option(s,a,'nlc.imode',4)
   apm_option(s,a,'nlc.nodes',3)
   apm_info(s,a,'SV','y')
   apm_info(s,a,'FV','u')
   apm_option(s,a,'u.fstatus',1)
   msg = 'Successful simulator initialization'
   return msg
   
def mhe_init(s,a):
   apm(s,a,'clear all')
   apm_load(s,a,'model.apm')
   csv_load(s,a,'data.csv')
   apm_option(s,a,'nlc.imode',5)
   apm_option(s,a,'nlc.nodes',3)
   apm_option(s,a,'nlc.web_plot_freq',1)
   apm_info(s,a,'FV','K')
   apm_info(s,a,'FV','tau')
   apm_info(s,a,'MV','u')
   apm_info(s,a,'CV','y')
   # status, whether the optimizer can use it
   apm_option(s,a,'K.status',1)
   apm_option(s,a,'tau.status',1)
   apm_option(s,a,'u.status',0)
   apm_option(s,a,'K.dcost',0.01)
   apm_option(s,a,'tau.dcost',0.01)
   apm_option(s,a,'K.dmax',10)
   apm_option(s,a,'tau.dmax',10)
   # feedback status
   apm_option(s,a,'K.fstatus',0)
   apm_option(s,a,'tau.fstatus',0)
   apm_option(s,a,'u.fstatus',1)
   apm_option(s,a,'y.fstatus',1)
   msg = 'Successful estimator initialization'   
   return msg
   
def sim(s,a,u):
   apm_meas(s,a,'u',u)
   apm(s,a,'solve')
   y = apm_tag(s,a,'y.model')
   return y

def mhe(s,a,inputs):
   y_meas = inputs[0]
   u_meas = inputs[1]
   apm_meas(s,a,'y',y_meas)
   apm_meas(s,a,'u',u_meas)
   apm(s,a,'solve')
   z = apm_sol(s,a)
   K = z['k'][-1]
   tau = z['tau'][-1]
   y_pred = z['y'][-1]
   return [K,tau,y_pred]   

# Number of cycles to run
cycles = 50

# Pulse in the process input
u = 5

# Server
s = 'http://byu.apmonitor.com'
# Application names
a1 = 'sim' + str(int(random.random()*10000))
a2 = 'mhe' + str(int(random.random()*10000))
# Initialize applications
print(sim_init(s,a1))
print(mhe_init(s,a2))

# Run cycles
for i in range(cycles):
    # Display cycle
    print('Cycle: ' + str(i+1))
    
    ## Change setpoint
    if i==19:
        u = 10
    elif i==39:
        u = 0
    
    ## Process simulator
    measurement_noise = 0.1*(random.random()-0.5)
    y_meas = sim(s,a1,u) + measurement_noise
    
    ## Moving Horizon Estimator (MHE)
    [k_pred,tau_pred,y_pred] = mhe(s,a2,[y_meas,u])
    	
    if (i==0):
       # Web viewers to see solution progression
       apm_web(s,a2) # Estimator
